using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Sockets;
using System.Net;
using System.Collections;

namespace LControl
{
	public delegate void ShowMsgDele(Object sender,ShowMsgEventArgs args);

	public class FTPClient
	{
		private Socket host_sock;
		public event ShowMsgDele ShowMsgEvent = null;
		private FTPDataTransMode _DataTransMode = FTPDataTransMode.Pasv;
		private static int Data_Port = 10000;

		public FTPClient()
		{
		}

		public void Connect(string host, int port, string user, string password)
		{
			host_sock = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

			//ӵ
			host_sock.Connect(host, port);
			try
			{

				if (user == null || user == "") throw new FTPClientException("ûΪ!");
					//½
					SendString(host_sock, String.Format("USER {0}\r\n", user), "GBK");
					SendString(host_sock, String.Format("PASS {0}\r\n", password), "GBK");

				host_sock.ReceiveTimeout = 1000;
				byte[] buf = new byte[host_sock.ReceiveBufferSize];
				int ret = 0, length = 0;
				do
				{
					length += ret;
					try
					{
						ret = host_sock.Receive(buf, length, host_sock.ReceiveBufferSize - length, SocketFlags.None);
					}
					catch (SocketException)
					{
						break;
					}
				} while (ret > 0);

				String msg = Encoding.GetEncoding("GBK").GetString(buf, 0, length);
				FTPReply reply = new FTPReply(msg);
				if (reply.Status / 100 >= 4) throw new FTPLoginFailedException(reply.Message);
				if (ShowMsgEvent != null) ShowMsgEvent(this, new ShowMsgEventArgs(msg));

				host_sock.ReceiveTimeout = 0;
				host_sock.Blocking = true;
			}
			catch (Exception)
			{
				host_sock.Close();
				throw;
			}
		}

		public void Close()
		{
			if (host_sock!=null && host_sock.Connected) host_sock.Close();
		}

		static public void SendString(Socket sock,String str,String EncodeName)
		{
			sock.Send(Encoding.GetEncoding(EncodeName).GetBytes(str));
		}

		private FTPReply GetReply()
		{
			byte[] buf = new byte[2048];
			int len=0;
			host_sock.ReceiveTimeout = 200;
			try
			{
				len = host_sock.Receive(buf, 0, 2048, SocketFlags.None);
			}
			catch (SocketException)
			{
			}

			if (len > 0)
			{
				String msg = Encoding.GetEncoding("GBK").GetString(buf, 0, len);
				if (ShowMsgEvent != null) ShowMsgEvent(this, new ShowMsgEventArgs(msg));
				FTPReply reply = new FTPReply(msg);
				if (reply.Status >= 400) throw new FTPStatusException(reply.Message);
				return reply;
			}
			else
			{
				throw new FTPNoReplyException("");
			}
		}

		public void CWD(String Dir)
		{
			SendString(host_sock, string.Format("CWD {0}\r\n", Dir), "GBK");
			GetReply();
		}

		public FTPFileInfoCollections LIST(String path)
		{
			string files_str = "";
			if (_DataTransMode == FTPDataTransMode.Pasv)
			{
				string host;
				int port;
				PASV(out host, out port);
			}
			else
			{
				Socket data_host=PORT();
				try
				{
					SendString(host_sock, "TYPE A\r\n", "GBK");
					GetReply();

					SendString(host_sock, string.Format("LIST {0}\r\n", path), "GBK");
					Socket data_sock = data_host.Accept();
					GetReply();
					try
					{
						byte[] buf = new byte[4096];
						data_sock.ReceiveTimeout = 1000;
						int ret;
						do
						{
							try
							{
								ret = data_sock.Receive(buf, 4096, SocketFlags.None);
							}
							catch (SocketException)
							{
								break;
							}
							if (ret > 0) files_str += Encoding.GetEncoding("GBK").GetString(buf, 0, ret);
						} while (ret > 0);
					}
					catch (Exception)
					{
						data_sock.Close();
						throw;
					}
					data_sock.Close();
					try
					{
						GetReply();
					}
					catch (FTPNoReplyException)
					{
					}
				}
				catch (Exception)
				{
					data_host.Close();
					throw;
				}
				data_host.Close();
			}
			return GetFileInfo(files_str);
		}

		private Socket PORT()
		{
			IPHostEntry host_entry = Dns.GetHostEntry(Dns.GetHostName());
			IPAddress host_ip = host_entry.AddressList[0];
			SendString(host_sock, string.Format("PORT {0},{1},{2}\r\n", host_ip.ToString().Replace('.', ','), Data_Port / 0x100, Data_Port % 0x100), "GBK");
			FTPReply reply = GetReply();
			Socket data_host = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
			data_host.Bind(new IPEndPoint(host_ip, Data_Port++));
			data_host.Listen(5);
			return data_host;
		}

		private void PASV(out string host, out int port)
		{
			SendString(host_sock, "PASV", "GBK");
			FTPReply reply = GetReply();
			if (reply != null)
			{
				int start, end;
				start = reply.Message.LastIndexOf("(");
				end = reply.Message.LastIndexOf(")");
				if (end > start)
				{
					string addr = reply.Message.Substring(start + 1, end - start - 1);
					string[] nums = addr.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
					if (nums.Length >= 6)
					{
						port = Convert.ToInt32(nums[4]) * 0x100 + Convert.ToInt32(nums[5]);
						host = string.Format("{0}.{1}.{2}.{3}", nums[0], nums[1], nums[2], nums[3]);
					}
					else
						throw new FTPClientException("");
				}
				else
				{
					throw new FTPClientException("");
				}
			}
			else
				throw new FTPClientException("");
		}

		public FTPFileInfoCollections GetFileInfo(String fistr)
		{
			string[] lines = fistr.Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
			if (lines.Length > 0)
			{
				FTPFileInfoCollections fic = new FTPFileInfoCollections();
				foreach (string line in lines)
				{
					string[] cols = line.Split(new char[] { ' ' }, 9, StringSplitOptions.RemoveEmptyEntries);
					if (cols.Length == 9)
					{
						if (cols[0][0] == 'd')
							fic.Add(new FTPFileInfo(cols[8], Convert.ToInt64(cols[4]), cols[0], true));
						else
							fic.Add(new FTPFileInfo(cols[8], Convert.ToInt64(cols[4]), cols[0], false));
					}
				}
				return fic;
			}
			else
				throw new FTPNoFileInfoException("");

		}

		public FTPDataTransMode DataTransMode
		{
			get { return _DataTransMode; }
			set { _DataTransMode = value; }
		}
	}

	public class ShowMsgEventArgs : EventArgs
	{
		public String Msg;

		public ShowMsgEventArgs(String msg):base()
		{
			Msg = msg;
		}
	}

	public class FTPReply
	{
		private int _Status;
		private string _Msg;
		private string _Reply;

		public int Status { get { return _Status; } }
		public string Reply { get { return _Reply; } }
		public string Message { get { return _Msg; } }

		public FTPReply(String replystr)
		{
			string[] lines = replystr.Split(new string[1] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
			_Reply = lines[lines.Length - 1];
			int index = _Reply.IndexOf(" ");
			if (index != -1)
			{
				_Status = Convert.ToInt32(_Reply.Substring(0, index));
				if(index+1<_Reply.Length) _Msg = _Reply.Substring(index + 1);
			}
		}
	}

	public class FTPLoginFailedException : Exception
	{
		public FTPLoginFailedException(string msg)
			: base(msg)
		{
		}
	}

	public class FTPClientException : Exception
	{
		public FTPClientException(string msg)
			: base(msg)
		{
		}
	}

	public class FTPNoReplyException : Exception
	{
		public FTPNoReplyException(string msg)
			: base(msg)
		{
		}
	}

	public class FTPNoFileInfoException : Exception
	{
		public FTPNoFileInfoException(string msg)
			: base(msg)
		{
		}
	}

	public class FTPStatusException : Exception
	{
		public FTPStatusException(string msg)
			: base(msg)
		{
		}
	}

	public class FTPFileInfo
	{
		public long Size;
		public string Name;
		public string Attr;
		public bool IsDirectory;

		public FTPFileInfo(string name, long size, string attr,bool isdir)
		{
			Name = name;
			Size = size;
			Attr = attr;
			IsDirectory = isdir;
		}
	}

	public class FTPFileInfoCollections
	{
		private ArrayList AllFiles;

		public FTPFileInfoCollections()
		{
			AllFiles = new ArrayList();
		}

		public void Add(FTPFileInfo fi)
		{
			AllFiles.Add(fi);
		}

		public FTPFileInfo this[int index]
		{
			get
			{
				return (FTPFileInfo)AllFiles[index];
			}
		}

		public void Clear()
		{
			AllFiles.Clear();
		}

		public int Length
		{
			get { return AllFiles.Count; }
		}

	}
	public enum FTPDataTransMode
	{
		Port, Pasv
	}
}
