using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace LControl
{
	public partial class FTPFileBrowser : FileBrowser
	{
		private FTPClient ftpClient;
		public event ShowMsgDele ShowMsgEvent = null;
		public FTPFileBrowser()
		{
			InitializeComponent();
			this.RootPath = "/";
			this.RootText = "/";
			this.CurrentPath = "/";
			ftpClient = new FTPClient();
			ftpClient.ShowMsgEvent += new ShowMsgDele(this.this_ShowMsg);
		}

		private void this_ShowMsg(Object sender, ShowMsgEventArgs args)
		{
			if (this.ShowMsgEvent != null) this.ShowMsgEvent(sender, args);
		}

		public void Connect(string host, int port, string user, string password,FTPDataTransMode mode)
		{
			ftpClient.Connect(host, port, user, password);
			ftpClient.DataTransMode = FTPDataTransMode.Port;
		}

		public void Close()
		{
			ftpClient.Close();
		}

		protected override void GetDirectionsAndFiles(string path, string filter, out LDirectoryInfo[] Dirs, out LFileInfo[] Files)
		{
			FTPFileInfoCollections fic;
			try
			{
				ftpClient.CWD(path);
				fic = ftpClient.LIST("");
			}
			catch (Exception ex)
			{
				throw new GetDirsAndFilesException(ex.Message);
			}

			int dir_count = 0, file_count = 0;
			for (int i = 0; i < fic.Length; i++)
			{
				if (fic[i].IsDirectory) dir_count++; else file_count++;
			}
			Dirs = new LDirectoryInfo[dir_count];
			Files = new LFileInfo[file_count];
			dir_count = 0;
			file_count = 0;
			for (int i = 0; i < fic.Length; i++)
			{
				string fullname;
				if (path[path.Length - 1] == '/') fullname = path + fic[i].Name; else fullname = path + "/" + fic[i].Name;
				if (fic[i].IsDirectory)
				{
					Dirs[dir_count++] = new LDirectoryInfo(FileAttributes.Normal, DateTime.Now, DateTime.Now, DateTime.Now, fic[i].Name,fullname);
				}
				else
				{
					Files[file_count++] = new LFileInfo(FileAttributes.Normal, DateTime.Now, DateTime.Now, DateTime.Now, fic[i].Size, fic[i].Name, fullname);
				}
			}
		}

		protected override void GetSubDirectorys(string path, out LDirectoryInfo[] Dirs)
		{

			FTPFileInfoCollections fic;
			try
			{
				fic = ftpClient.LIST(path);
			}
			catch (Exception ex)
			{
				throw new GetSubDirsException(ex.Message);
			}

			int dir_count = 0;
			for (int i = 0; i < fic.Length; i++)
			{
				if (fic[i].IsDirectory) dir_count++;
			}
			Dirs = new LDirectoryInfo[dir_count];
			dir_count = 0;
			for (int i = 0; i < fic.Length; i++)
			{
				string fullname;
				if (path[path.Length - 1] == '/') fullname = path + fic[i].Name; else fullname = path + "/" + fic[i].Name;
				if (fic[i].IsDirectory)
				{
					Dirs[dir_count++] = new LDirectoryInfo(FileAttributes.Normal, DateTime.Now, DateTime.Now, DateTime.Now, fic[i].Name, fullname);
				}
			}
		}
	}
}
