using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace LControl
{
    public delegate void GetAllDriveDelegate(out LDriveInfo[] di);
    public delegate void GetSubDirectoryDelegate(string path, out LDirectoryInfo[] di);
    public delegate void AfterSelectEventHandler(object sender,FolderBrowserEventArgs e);

    public partial class FolderBrowser : UserControl
    {
        private string _SelectedPath = "";
        private string _RootText = "";
        private LPath _RootPath = new LPath("");
        private bool _ShowHiddenDir = false;

        public GetAllDriveDelegate GetAllDriveDele = null;
        public GetSubDirectoryDelegate GetSubDirectoryDele = null;

        [Browsable(true),Category("Ϊ"),Description("ѡ·ʱ")]
        public event AfterSelectEventHandler AfterSelect;

        public FolderBrowser()
        {
            InitializeComponent();

			Image16.Images.Add(LResource.computer16);
			Image16.Images.Add(LResource.disk16);
			Image16.Images.Add(LResource.folder16);
			Image16.Images.Add(LResource.folder_open16);

            TreeNode tn = new TreeNode(_RootText, 0, 0);
            tn.Tag = new TreeNodeTag("", NodeKind.Computer, false);
            tn.Name = "";
            FolderTree.Nodes.Add(tn);
            FolderTree.Nodes[0].Nodes.Add(".");
            this.FolderTree.Nodes[0].Text = _RootText;
        }

        private void RefreshSubNodes(TreeNode Node)
        {
            TreeNodeTag tag = (TreeNodeTag)Node.Tag;
            Node.Nodes.Clear();
            if (tag.Kind == NodeKind.Computer)
            {
                LDriveInfo[] ldis;
                GetAllDrive(out ldis);
                foreach (LDriveInfo di in ldis)
                {
                    TreeNode tn = new TreeNode(di.VolumeLabel + "(" + di.Name.Remove(2) + ")", 1, 1);
                    tn.Tag = new TreeNodeTag(di.Name, NodeKind.Drive, false);
                    tn.Name = di.Name.Remove(2);
                    tn.Nodes.Add(".");
                    Node.Nodes.Add(tn);
                }
            }
            else
            {
                LDirectoryInfo[] ldis;
                GetSubDirectory(tag.FullPath, out ldis);
                foreach (LDirectoryInfo di in ldis)
                {
                    if (!(_ShowHiddenDir == false && (di.Attributes & System.IO.FileAttributes.Hidden) == System.IO.FileAttributes.Hidden))
                    {
                        TreeNode tn = new TreeNode(di.Name, 2, 3);
                        tn.Tag = new TreeNodeTag(di.FullName, NodeKind.Folder, false);
                        tn.Name = di.Name;
                        tn.Nodes.Add(".");
                        Node.Nodes.Add(tn);
                    }
                }
            }
            tag.Refreshed = true;
        }

        private void FolderTree_BeforeExpand(object sender, TreeViewCancelEventArgs e)
        {
            TreeNodeTag tag = (TreeNodeTag)e.Node.Tag;
            if (tag.Refreshed == false)
            {
                try
                {
                    RefreshSubNodes(e.Node);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(this, ex.Message, LControl_Language.strErr1);
                }
            }
        }

        private void FolderTree_AfterCollapse(object sender, TreeViewEventArgs e)
        {
            TreeNodeTag tag = (TreeNodeTag)e.Node.Tag;
            tag.Refreshed = false;
        }

        private void FolderTree_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (this.AfterSelect != null) AfterSelect(this, new FolderBrowserEventArgs(((TreeNodeTag)e.Node.Tag).FullPath));
        }

        private void FolderBrowser_Load(object sender, EventArgs e)
        {
            if (this.DesignMode == false)
            {
                SelectedPath = _SelectedPath;
            }
        }

        #region з˵

        [Browsable(true), Description("·"), Category("")]
        public string RootPath
        {
            get
            {
                return _RootPath.Path;
            }
            set
            {
                _RootPath = new LPath(value);
                _SelectedPath = value;

                FolderTree.Nodes.Clear();

                if (value == "")
                {
                    TreeNode tn = new TreeNode(_RootText, 0, 0);
                    tn.Tag = new TreeNodeTag("", NodeKind.Computer, false);
                    tn.Name = value;
                    FolderTree.Nodes.Add(tn);
                }
                else
                {
                    TreeNode tn = new TreeNode(_RootText, 2, 3);
                    tn.Tag = new TreeNodeTag(value, NodeKind.Folder, false);
                    tn.Name = value;
                    FolderTree.Nodes.Add(tn);
                }
                FolderTree.Nodes[0].Nodes.Add(".");
                this.FolderTree.Nodes[0].Text = _RootText;
                if (this.Created)
                {
                    SelectedPath = value;
                }
            }
        }

        [Browsable(true), Description("ǰѡ·"), Category("")]
        public string SelectedPath
        {
            get
            {
                return _SelectedPath;
            }
            set
            {
                try
                {
                    LPath path;
                    if (value == "")
                        path = _RootPath;
                    else
                        path = new LPath(value);

                    if (this.DesignMode)
                    {
                        if (_RootPath != path && !_RootPath.IsParent(path))
                            throw new Exception(LControl_Language.strDesignTip1);
                    }
                    else
                    {
                        if (this.Created)
                        {
                            if (path == _RootPath || path.Path == "")
                            {
                                FolderTree.SelectedNode = FolderTree.Nodes[0];
                            }
                            else if (_RootPath.IsParent(path))
                            {
                                int start = 0;
                                LPathNode[] pns = path.Nodes;
                                TreeNode tn = FolderTree.Nodes[0];

                                for (start = _RootPath.Length; start < pns.Length; start++)
                                {
                                    if (((TreeNodeTag)tn.Tag).Refreshed == false)
                                    {
                                        RefreshSubNodes(tn);
                                        tn = tn.Nodes[pns[start].Value];
                                        if (tn == null) throw new PathNotFoundException(LControl_Language.strInvalidPath);
                                    }
                                    else
                                    {
                                        if (tn.Nodes[pns[start].Value] == null)
                                        {
                                            RefreshSubNodes(tn);
                                            if (tn.Nodes[pns[start].Value] == null)
                                                throw new PathNotFoundException(LControl_Language.strInvalidPath);
                                            else
                                                tn = tn.Nodes[pns[start].Value];
                                        }
                                        else
                                            tn = tn.Nodes[pns[start].Value];
                                    }
                                }
                                FolderTree.SelectedNode = tn;
                            }
                            else
                                throw new PathNotFoundException(LControl_Language.strInvalidPath);
                        }
                    }
                    _SelectedPath = path.Path;
                }
                catch (Exception)
                {
                    throw;
                }
            }
        }

        [Browsable(true), Description("Ŀ¼"), Category("")]
        public string RootText
        {
            get
            {
                return _RootText;
            }
            set
            {
                _RootText = value;
                if (this.FolderTree.Nodes.Count != 0) this.FolderTree.Nodes[0].Text = value;
            }
        }


        [Browsable(true), Description("Ƿʾļ"), Category("Ϊ")]
        public bool ShowHiddenDir
        {
            get { return _ShowHiddenDir; }
            set { _ShowHiddenDir = value; }
        }

        #endregion

        #region ҪʵFolderBrowser,д麯(дʱûຯ)

        //ָĿ¼Ŀ¼ͨDirs(Ŀ¼)
        //д˵
        //(1)д÷
        //(2)ȡĿ¼ļʧʱ׳쳣GetSubDirsException
        //:
        //(1)path[in]:ҪȡĿ¼ϢĿ¼·;
        //(2)filter[in]:ɸѡַ
        //(3)Dirs[out]:ڷĿ¼
        virtual protected void GetSubDirectory(string path, out LDirectoryInfo[] di)
        {
            if (this.GetSubDirectoryDele != null)
                this.GetSubDirectoryDele(path, out di);
            else
                di = null;
        }

        /// <summary>
        ///ϵ߼
        ///ֵLDriveInfo飬ʾϵ߼
        ///д˵
        ///(1)Ŀ¼Ϊҵĵʱ,д÷
        ///(2)ȡ߼ʧʱ׳쳣GetDrivesException
        /// </summary>
        virtual protected void GetAllDrive(out LDriveInfo[] di)
        {
            if (this.GetAllDriveDele != null) this.GetAllDriveDele(out di); else di = null;
        }

        #endregion
    }

    public class TreeNodeTag
    {
        public string FullPath;
        public NodeKind Kind;
        public bool Refreshed;

        public TreeNodeTag(string path, NodeKind kind, bool refreshed)
        {
            FullPath=path;
            Kind=kind;
            Refreshed=refreshed;
        }

    }

    public class FolderBrowserEventArgs : EventArgs
    {
        private string _Path;

        public string Path
        {
            get
            {
                return _Path;
            }
        }

        public FolderBrowserEventArgs(string path)
        {
            _Path = path;
        }
    }

    public class LPathNode
    {
        private string _Value;
        private string _FullPath;

        public LPathNode(string value, string fullpath)
        {
            _Value = value;
            if (fullpath[fullpath.Length - 1] == ':')
                _FullPath = fullpath + '\\';
            else
                _FullPath = fullpath;
        }

        public string Value
        {
            get
            {
                return _Value;
            }
        }

        public string FullPath
        {
            get
            {
                return _FullPath;
            }
        }
    }

    public class LPath
    {
        private string _Path = "";
        private ArrayList _Nodes=new ArrayList();

        public LPath(string path)
        {
            Path = path;
        }

        public int Length
        {
            get
            {
                return _Nodes.Count;
            }
        }

        public string Path
        {
            get
            {
                return _Path;
            }
            set
            {
				if (value != "")
				{
					if (value[0] == '/')
					{
						char[] psp = new char[1] { '/' };

						_Nodes.Clear();
						_Nodes.Add(new LPathNode("", "/"));
						string[] ps = value.Split(psp, StringSplitOptions.RemoveEmptyEntries);
						if (ps.Length == 0)
							_Path = "/";
						else
						{
							_Path = "";
							foreach (string s in ps) _Nodes.Add(new LPathNode(s, _Path += '/' + s));
						}
					}
					else
					{
						char[] psp = new char[2] { '\\', '/' };
						string[] ps = value.Split(psp, StringSplitOptions.RemoveEmptyEntries);

						_Nodes.Clear();
						_Path = "";
						foreach (string s in ps)
						{
							if (_Path != "")
								_Nodes.Add(new LPathNode(s, _Path += '\\' + s));
							else
								_Nodes.Add(new LPathNode(s, _Path = s));
						}
						if (_Path != "" && _Path[_Path.Length - 1] == ':') _Path += "\\";
					}

				}
				else
				{
					_Nodes.Clear();
					_Path = "";
				}
            }
        }

        public LPathNode[] Nodes
        {
            get
            {
                LPathNode[] pns = new LPathNode[_Nodes.Count];

                int i = 0;
                foreach (LPathNode pn in _Nodes)
                {
                    pns[i++] = pn;
                }

                return pns;
            }
        }

        public static bool operator ==(LPath p1, LPath p2)
        {
            return p1.Equals(p2);
        }

        public static bool operator !=(LPath p1, LPath p2)
        {
            return !p1.Equals(p2);
        }

        public override bool Equals(object obj)
        {
            if (obj.GetType() != typeof(LPath))
                return false;
            else
            {
                LPath p = (LPath)obj;

                if (this._Nodes.Count != p._Nodes.Count)
                    return false;
                else
                {
                    int i = 0;
                    for (i = 0; i < this._Nodes.Count; i++)
                    {
                        if (string.Compare(((LPathNode)_Nodes[i]).Value, ((LPathNode)p._Nodes[i]).Value, true) != 0) return false;
                    }
                    return true;
                }
            }
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        public bool IsParent(LPath path)
        {
            if (path.Length <= this.Length)
                return false;
            else
            {
                int i=0;
                foreach(LPathNode pn in _Nodes)
                {
                    if (string.Compare(pn.Value, ((LPathNode)path._Nodes[i++]).Value, true) != 0) return false;
                }
                return true;
            }
        }

        public bool Upward()
        {
            if (_Nodes.Count != 0)
            {
                _Nodes.RemoveAt(_Nodes.Count - 1);
                if (_Nodes.Count == 0)
                    _Path = "";
                else
                    _Path = ((LPathNode)_Nodes[_Nodes.Count - 1]).FullPath;
                return true;
            }
            else
                return false;
        }
    }
}
